// src/index.mts
import { EditorState } from "@codemirror/state";
import { EditorView as EditorView2 } from "@codemirror/view";
import { minimalSetup } from "codemirror";

// src/stickToBottom.mts
import {
  ViewPlugin
} from "@codemirror/view";
var PIXEL_BUFFER = 100;
var StickToBottomViewPlugin = class {
  constructor(view) {
    this._animationFrameRequest = null;
    this._stickToBottom = false;
  }
  update(update) {
    if (this._animationFrameRequest != null || !update.viewportChanged) {
      return;
    }
    if (!this._stickToBottom) {
      this._stickToBottom = Math.abs(
        update.view.scrollDOM.scrollHeight - Math.round(update.view.scrollDOM.scrollTop) - update.view.scrollDOM.clientHeight
      ) < PIXEL_BUFFER;
      if (!this._stickToBottom) {
        return;
      }
    }
    const onScroll = () => {
      if (this._animationFrameRequest == null) {
        return;
      }
      cancelAnimationFrame(this._animationFrameRequest);
      this._animationFrameRequest = null;
      this._stickToBottom = false;
    };
    this._animationFrameRequest = requestAnimationFrame(() => {
      update.view.scrollDOM.removeEventListener("scroll", onScroll);
      update.view.scrollDOM.scroll({
        top: update.view.scrollDOM.scrollHeight,
        behavior: "instant"
      });
      this._animationFrameRequest = null;
      this._stickToBottom = true;
    });
    update.view.scrollDOM.addEventListener("scroll", onScroll);
  }
};
function getStickToBottomViewPlugin() {
  const plugin = ViewPlugin.fromClass(StickToBottomViewPlugin);
  return plugin;
}

// src/addConsoleError.mts
import { StateField, StateEffect } from "@codemirror/state";
import { Decoration, EditorView } from "@codemirror/view";
var addConsoleError = StateEffect.define({
  map: ({ from, to }, change) => {
    return {
      from: change.mapPos(from),
      to: change.mapPos(to)
    };
  }
});
var consoleErrorField = StateField.define({
  create() {
    return Decoration.none;
  },
  update(errors, tr) {
    errors = errors.map(tr.changes);
    for (const effect of tr.effects)
      if (effect.is(addConsoleError)) {
        errors = errors.update({
          add: [consoleErrorMark.range(effect.value.from, effect.value.to)]
        });
      }
    return errors;
  },
  provide: (field) => EditorView.decorations.from(field)
});
var consoleErrorMark = Decoration.mark({ class: "cm-console-error" });
var consoleErrorTheme = EditorView.baseTheme({
  ".cm-console-error": { color: "red" }
});
function consoleErrorLine(view, lineNumber) {
  const line = view.state.doc.line(lineNumber);
  const effects = [
    addConsoleError.of({ from: line.from, to: line.to })
  ];
  if (view.state.field(consoleErrorField, false) == null) {
    effects.push(
      StateEffect.appendConfig.of([consoleErrorField, consoleErrorTheme])
    );
  }
  view.dispatch({ effects });
  return true;
}

// src/index.mts
function console() {
  const stickToBottom = getStickToBottomViewPlugin();
  return [
    minimalSetup,
    EditorState.readOnly.of(true),
    stickToBottom,
    EditorView2.lineWrapping
  ];
}
export {
  console,
  consoleErrorLine
};
//# sourceMappingURL=index.mjs.map
